@@ -3,6 +3,7 @@ npudist <-
              "please set 'bws'")), ...){
     args = list(...)
     
+if (is.recursive(bws)){
     if (!is.null(bws$formula) && is.null(args$tdat))
       UseMethod("npudist",bws$formula)
     else if (!is.null(args$data) || !is.null(args$newdata))
@@ -11,6 +12,9 @@ npudist <-
       UseMethod("npudist",bws$call)
     else
       UseMethod("npudist",bws)
+} else {
+      UseMethod("npudist",bws)
+}
 
   }
 
@@ -86,10 +90,10 @@ npudist.bandwidth <-
     ## re-assign levels in training and evaluation data to ensure correct
     ## conversion to numeric type.
     
-    tdat <- relevel(tdat, bws$xdati)
+    tdat <- adjustLevels(tdat, bws$xdati)
     
     if (!no.e)
-      edat <- relevel(edat, bws$xdati)
+      edat <- adjustLevels(edat, bws$xdati)
 
     ## grab the evaluation data before it is converted to numeric
     if(no.e)
@@ -164,11 +168,10 @@ npudist.default <-
 
     tdat <- toFrame(tdat)
 
-    tbw = bandwidth( bws,
-      ...,
-      nobs = 0,
-      xdati = untangle(tdat),
-      xnames = names(tdat) )
+    tbw <- npudensbw(bws = bws,
+                     dat = tdat,
+                     bandwidth.compute = FALSE,
+                     ...)
 
     eval(parse(text=paste("npudist.bandwidth(tdat = tdat, bws = tbw",
                  ifelse(missing(edat), "",", edat = edat"), ")")))
